import { dataSource } from "../../data-source";
import { role } from "../../entity/user/role";
import { user } from "../../entity/user/user";
import InternalException from "../../exceptions/internalException";

export default abstract class UserService {
  /**
   * @description get users
   * @returns {Promise<Array<user>>}
   */
  static async getAll(): Promise<Array<user>> {
    return await dataSource
      .getRepository(user)
      .createQueryBuilder("user")
      .leftJoinAndSelect("user.roles", "roles")
      .leftJoinAndSelect("user.permissions", "permissions")
      .leftJoinAndSelect("roles.permissions", "role_permissions")
      .orderBy("firstname", "ASC")
      .addOrderBy("lastname", "ASC")
      .getMany()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new InternalException("users not found", err);
      });
  }

  /**
   * @description get user by id
   * @param id string
   * @returns {Promise<user>}
   */
  static async getById(id: string): Promise<user> {
    return await dataSource
      .getRepository(user)
      .createQueryBuilder("user")
      .leftJoinAndSelect("user.roles", "roles")
      .leftJoinAndSelect("user.permissions", "permissions")
      .leftJoinAndSelect("roles.permissions", "role_permissions")
      .where("user.id = :id", { id: id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new InternalException("user not found by id", err);
      });
  }

  /**
   * @description get user by username
   * @param username string
   * @returns {Promise<user>}
   */
  static async getByUsername(username: string): Promise<user> {
    return await dataSource
      .getRepository(user)
      .createQueryBuilder("user")
      .select()
      .where("user.username = :username", { username: username })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new InternalException("user not found by username", err);
      });
  }

  /**
   * @description get users by mail or username
   * @param username string
   * @param mail string
   * @returns {Promise<Array<user>>}
   */
  static async getByMailOrUsername(mail?: string, username?: string): Promise<Array<user>> {
    return await dataSource
      .getRepository(user)
      .createQueryBuilder("user")
      .select()
      .where("user.mail = :mail", { mail: mail })
      .orWhere("user.username = :username", { username: username })
      .getMany()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new InternalException("user not found by mail or username", err);
      });
  }

  /**
   * @description get count of users
   * @returns {Promise<number>}
   */
  static async count(): Promise<number> {
    return await dataSource
      .getRepository(user)
      .createQueryBuilder("user")
      .select()
      .getCount()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new InternalException("could not count users", err);
      });
  }

  /**
   * @description get roles assigned to user
   * @param userId string
   * @returns {Promise<Array<role>>}
   */
  static async getAssignedRolesByUserId(userId: string): Promise<Array<role>> {
    return await dataSource
      .getRepository(user)
      .createQueryBuilder("user")
      .leftJoinAndSelect("user.roles", "roles")
      .leftJoinAndSelect("roles.permissions", "role_permissions")
      .where("user.id = :id", { id: userId })
      .getOneOrFail()
      .then((res) => {
        return res.roles;
      })
      .catch((err) => {
        throw new InternalException("could not get roles for user", err);
      });
  }
}
